@@ -3,15 +3,16 @@ import os
 import subprocess
 import sys
 import textwrap
+from pathlib import Path
 from unittest import mock
 
-import pytest  # type: ignore
+import pytest  # type: ignore[import-not-found]
 
 import pipx.main
 import pipx.util
 from helpers import run_pipx_cli
 from package_info import PKG
-from pipx import constants
+from pipx import paths, shared_libs
 
 
 def test_help_text(pipx_temp_env, monkeypatch, capsys):
@@ -63,7 +64,7 @@ def test_cache(pipx_temp_env, monkeypatch, capsys, caplog):
 
 @mock.patch("os.execvpe", new=execvpe_mock)
 def test_cachedir_tag(pipx_ultra_temp_env, monkeypatch, capsys, caplog):
-    tag_path = constants.PIPX_VENV_CACHEDIR / "CACHEDIR.TAG"
+    tag_path = paths.ctx.venv_cache / "CACHEDIR.TAG"
     assert not tag_path.exists()
 
     # Run pipx to create tag
@@ -115,7 +116,7 @@ def test_run_script_from_internet(pipx_temp_env, capsys):
     ],
 )
 def test_appargs_doubledash(pipx_temp_env, capsys, monkeypatch, input_run_args, expected_app_with_args):
-    parser = pipx.main.get_command_parser()
+    parser, _ = pipx.main.get_command_parser()
     monkeypatch.setattr(sys, "argv", ["pipx", "run"] + input_run_args)
     parsed_pipx_args = parser.parse_args()
     pipx.main.check_args(parsed_pipx_args)
@@ -183,7 +184,7 @@ def test_run_without_requirements(caplog, pipx_temp_env, tmp_path):
         textwrap.dedent(
             f"""
                 from pathlib import Path
-                Path({repr(str(out))}).write_text({repr(test_str)})
+                Path({str(out)!r}).write_text({test_str!r})
             """
         ).strip()
     )
@@ -208,7 +209,7 @@ def test_run_with_requirements(caplog, pipx_temp_env, tmp_path):
                 import certifi
                 # Check the installed version
                 from pathlib import Path
-                Path({repr(str(out))}).write_text(requests.__version__)
+                Path({str(out)!r}).write_text(requests.__version__)
             """
         ).strip(),
         encoding="utf-8",
@@ -234,7 +235,7 @@ def test_run_with_requirements_old(caplog, pipx_temp_env, tmp_path):
                 import certifi
                 # Check the installed version
                 from pathlib import Path
-                Path({repr(str(out))}).write_text(requests.__version__)
+                Path({str(out)!r}).write_text(requests.__version__)
             """
         ).strip(),
         encoding="utf-8",
@@ -270,7 +271,7 @@ def test_run_with_args(caplog, pipx_temp_env, tmp_path):
             f"""
                 import sys
                 from pathlib import Path
-                Path({repr(str(out))}).write_text(str(int(sys.argv[1]) + 1))
+                Path({str(out)!r}).write_text(str(int(sys.argv[1]) + 1))
             """
         ).strip()
     )
@@ -291,7 +292,7 @@ def test_run_with_requirements_and_args(caplog, pipx_temp_env, tmp_path):
                 import packaging
                 import sys
                 from pathlib import Path
-                Path({repr(str(out))}).write_text(str(int(sys.argv[1]) + 1))
+                Path({str(out)!r}).write_text(str(int(sys.argv[1]) + 1))
             """
         ).strip()
     )
@@ -299,6 +300,21 @@ def test_run_with_requirements_and_args(caplog, pipx_temp_env, tmp_path):
     assert out.read_text() == "2"
 
 
+def test_pip_args_forwarded_to_shared_libs(pipx_ultra_temp_env, capsys, caplog):
+    # strategy:
+    # 1. start from an empty env to ensure the next command would trigger a shared lib update
+    assert shared_libs.shared_libs.needs_upgrade
+    # 2. install any package with --no-index
+    # and check that the shared library update phase fails
+    return_code = run_pipx_cli(["run", "--verbose", "--pip-args=--no-index", "pycowsay", "hello"])
+    assert "Upgrading shared libraries in" in caplog.text
+
+    captured = capsys.readouterr()
+    assert return_code != 0
+    assert "ERROR: Could not find a version that satisfies the requirement pip" in captured.err
+    assert "Failed to upgrade shared libraries" in caplog.text
+
+
 @mock.patch("os.execvpe", new=execvpe_mock)
 def test_run_with_invalid_requirement(capsys, pipx_temp_env, tmp_path):
     script = tmp_path / "test.py"
@@ -328,7 +344,7 @@ def test_run_script_by_absolute_name(caplog, pipx_temp_env, tmp_path):
         textwrap.dedent(
             f"""
                 from pathlib import Path
-                Path({repr(str(out))}).write_text({repr(test_str)})
+                Path({str(out)!r}).write_text({test_str!r})
             """
         ).strip()
     )
@@ -345,7 +361,7 @@ def test_run_script_by_relative_name(caplog, pipx_temp_env, monkeypatch, tmp_pat
         textwrap.dedent(
             f"""
                 from pathlib import Path
-                Path({repr(str(out))}).write_text({repr(test_str)})
+                Path({str(out)!r}).write_text({test_str!r})
             """
         ).strip()
     )
@@ -365,7 +381,7 @@ def test_run_with_windows_python_version(caplog, pipx_temp_env, tmp_path):
             f"""
                 import sys
                 from pathlib import Path
-                Path({repr(str(out))}).write_text(sys.version)
+                Path({str(out)!r}).write_text(sys.version)
             """
         ).strip()
     )
@@ -386,3 +402,15 @@ def test_run_shared_lib_as_app(pipx_temp_env, monkeypatch, capfd):
     run_pipx_cli_exit(["run", "pip", "--help"])
     captured = capfd.readouterr()
     assert "pip <command> [options]" in captured.out
+
+
+@mock.patch("os.execvpe", new=execvpe_mock)
+def test_run_local_path_entry_point(pipx_temp_env, caplog, root):
+    empty_project_path = (Path("testdata") / "empty_project").as_posix()
+    os.chdir(root)
+
+    caplog.set_level(logging.INFO)
+
+    run_pipx_cli_exit(["run", empty_project_path])
+
+    assert "Using discovered entry point for 'pipx run'" in caplog.text
